# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-26 19:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pushy', '0004_auto_20160220_1828'),
    ]

    operations = [
        migrations.AlterField(
            model_name='device',
            name='key',
            field=models.CharField(max_length=255),
        ),
        migrations.AlterUniqueTogether(
            name='device',
            unique_together=set([('key', 'type')]),
        ),
    ]
